/*
 * CSE 154 Summer 2019
 * API for Fake Piazza
 *
 * An API that can log users in to a fake Piazza service and provide statistics on their
 * fake usage of the service.
 * *** Endpoint Documentation ***
 * Endpoint: /login
 * Description: Logs the user into the service with given credentials.
 * Request Type: POST
 * Response Type: Plain Text
 * Required Parameters: username, password
 * Example Request: /login {username: "Dubs", password: "boundless"}
 * Example Response: Successfully logged in!
 * Error Handling: Responds with 400 plain text if they do not give required params or they
 *                 provide invalid credentials.
 * *************************************
 * Endpoint: /logout
 * Description: Logs a user out if they are logged in.
 * Request Type: POST
 * Response Type: Plain Text
 * Example Request: /logout
 * Example Response: Successfully logged out!
 * *************************************
 * Endpoint: /stats
 * Description: Provides usage statistics for the logged in individual, if logged in.
 * Request Type: GET
 * Response Type: JSON
 * Example Request: /stats
 * Example Response:
 * {
 *   "Username": "Dubs",
 *   "Questions Asked": 20,
 *   "Courses": [
 *                "CSE 154",
 *                "MATH 126"
 *              ]
 *   "Responses Recieved": 11
 * }
 * Error Handling: Responds in text with 400 if the user is not logged in.
 */

"use strict";

const express = require("express");
const mysql = require("promise-mysql");
const multer = require("multer"); // Handles form-data requests.
const cookieParser = require("cookie-parser");
const app = express();

app.use(express.urlencoded({extended: true}));
app.use(express.json());
app.use(multer().none());
app.use(cookieParser());

const INVALID_PARAM_ERROR = 400;
const SERVER_ERROR = 500;
const SERVER_ERROR_MSG = "Something went wrong on the server.";

// TODO: Implement the POST endpoint /login

// TODO: Implement the POST endpoint /logout

// TODO: Implement the GET endpoint /stats

/**
 * Establishes a database connection and returns the database object.
 * @returns {Object} - The database object for the connection.
 */
async function getDB() {
  let db = await mysql.createConnection({
    host: "localhost",
    port: "YOUR PORT HERE.", // TODO
    user: "root",
    password: "root",
    database: "piazzadb"
  });
  return db;
}

/**
 * Generates an unused sessionid and returns it to the user.
 * @param {object} db - A connection object to the database.
 * @returns {string} - The random session id.
 */
async function getSessionId(db) {
  let query = "SELECT sessionid FROM users WHERE sessionid = ?";
  let id;
  do {
    // This wizardry comes from https://gist.github.com/6174/6062387
    id = Math.random().toString(36)
      .substring(2, 15) + Math.random().toString(36)
      .substring(2, 15);
  } while ((await db.query(query, [id])).length > 0);
  return id;
}

const PORT = process.env.PORT || 8000;
app.listen(PORT);
